import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:hkqr_fps/hkqr_fps.dart';
import 'package:qr_flutter/qr_flutter.dart';

class QrEncodePage extends StatefulWidget {
  const QrEncodePage({Key key}) : super(key: key);

  @override
  _QrEncodePageState createState() => _QrEncodePageState();
}

class _QrEncodePageState extends State<QrEncodePage> {
  final _key = GlobalKey();

  bool isDynamic = true;
  FpsType fpsType = FpsType.phone;
  bool amountEditable = true;
  bool isCurrencyHK = true;

  TextEditingController fpsValueController;

  TextEditingController amountController;

  TextEditingController timeoutController;

  TextEditingController clearingCodeController;

  TextEditingController billController;

  TextEditingController referenceController;

  @override
  void initState() {
    super.initState();
    fpsValueController = TextEditingController();
    amountController = TextEditingController();
    timeoutController = TextEditingController();
    clearingCodeController = TextEditingController();
    billController = TextEditingController();
    referenceController = TextEditingController();
  }

  @override
  void dispose() {
    fpsValueController.dispose();
    amountController.dispose();
    timeoutController.dispose();
    clearingCodeController.dispose();
    billController.dispose();
    referenceController.dispose();
    super.dispose();
  }

  Widget _buildPicker(String text) {
    return Container(
      margin: EdgeInsets.zero,
      padding: EdgeInsets.symmetric(horizontal: 12, vertical: 6),
      decoration: BoxDecoration(
        color: Theme.of(context).cardColor,
        border: Border.all(color: Theme.of(context).primaryColor, width: 0.5),
        borderRadius: BorderRadius.all(Radius.circular(5)),
      ),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [Text(text), Icon(Icons.arrow_drop_down)],
      ),
    );
  }

  List<Widget> get _divider => [
        SizedBox(height: 15),
        Divider(height: 2, color: Theme.of(context).primaryColor),
        SizedBox(height: 15),
      ];

  @override
  Widget build(BuildContext context) {
    final qrInfo = HKQrInfo(
      isDynamic: isDynamic,
      fpsProxy: FpsProxy(fpsType, fpsValueController.text),
      timeout: timeoutController.text,
      transactionAmountEditable: amountEditable,
      isCurrencyHK: isCurrencyHK,
      amount: amountController.text,
      clearingCode: clearingCodeController.text,
      bill: billController.text,
      reference: referenceController.text,
    );

    final encoded = qrInfo.encode();

    final fpsTypeDesc =
        fpsType == FpsType.phone ? 'Phone' : (fpsType == FpsType.email ? 'Email' : 'Fps ID');

    FpsType typeFromDesc(String desc) =>
        desc == 'Phone' ? FpsType.phone : (desc == 'Email' ? FpsType.email : FpsType.fpsId);

    final width = MediaQuery.of(context).size.width;
    return Scaffold(
      appBar: AppBar(title: Text('Encode to QR String')),
      body: SingleChildScrollView(
        child: Padding(
          padding: EdgeInsets.symmetric(horizontal: kIsWeb ? width * 0.3 : 36),
          child: Column(children: [
            SizedBox(height: 15),
            PopupMenuButton(
              initialValue: isDynamic ? 0 : 1,
              child: _buildPicker('${isDynamic ? 'Dynamic' : 'Static'} QR Code'),
              itemBuilder: (context) {
                return [
                  PopupMenuItem(value: 0, child: Text('Dynamic')),
                  PopupMenuItem(value: 1, child: Text('Static')),
                ];
              },
              onSelected: (value) => setState(() => isDynamic = value == 0),
            ),
            ..._divider,
            PopupMenuButton(
              initialValue: fpsTypeDesc,
              child: _buildPicker('Fps Type:    $fpsTypeDesc'),
              itemBuilder: (context) {
                return [
                  PopupMenuItem(value: 'Phone', child: Text('Phone')),
                  PopupMenuItem(value: 'Email', child: Text('Email')),
                  PopupMenuItem(value: 'Fps ID', child: Text('Fps ID')),
                ];
              },
              onSelected: (value) => setState(() => fpsType = typeFromDesc(value)),
            ),
            SizedBox(height: 15),
            TextField(
              decoration: InputDecoration(
                border: OutlineInputBorder(),
                labelText: fpsTypeDesc,
              ),
              controller: fpsValueController,
            ),
            ..._divider,
            PopupMenuButton(
              initialValue: isCurrencyHK ? 0 : 1,
              child: _buildPicker('Currency:    ${isCurrencyHK ? 'HKD' : 'RMB'}'),
              itemBuilder: (context) {
                return [
                  PopupMenuItem(value: 0, child: Text('HKD')),
                  PopupMenuItem(value: 1, child: Text('RMB')),
                ];
              },
              onSelected: (value) => setState(() => isCurrencyHK = value == 0),
            ),
            SizedBox(height: 15),
            TextField(
              decoration: InputDecoration(
                border: OutlineInputBorder(),
                labelText: 'Amount (${isCurrencyHK ? 'HKD' : 'RMB'})',
              ),
              keyboardType: TextInputType.number,
              controller: amountController,
            ),
            SizedBox(height: 15),
            PopupMenuButton(
              initialValue: amountEditable ? 0 : 1,
              child: _buildPicker(amountEditable ? 'Amount Editable' : 'Amount NOT Editable'),
              itemBuilder: (context) {
                return [
                  PopupMenuItem(value: 0, child: Text('Amount Editable')),
                  PopupMenuItem(value: 1, child: Text('Amount NOT Editable')),
                ];
              },
              onSelected: (value) => setState(() => amountEditable = value == 0),
            ),
            ..._divider,
            TextField(
              decoration: InputDecoration(
                border: OutlineInputBorder(),
                labelText: 'Time Out',
                hintText: 'yymmddhhmmss',
              ),
              keyboardType: TextInputType.datetime,
              controller: timeoutController,
            ),
            ..._divider,
            TextField(
              decoration: InputDecoration(
                border: OutlineInputBorder(),
                labelText: 'Clearing Code',
              ),
              controller: clearingCodeController,
            ),
            ..._divider,
            TextField(
              decoration: InputDecoration(
                border: OutlineInputBorder(),
                labelText: 'Bill Number',
              ),
              controller: billController,
            ),
            ..._divider,
            TextField(
              decoration: InputDecoration(
                border: OutlineInputBorder(),
                labelText: 'Reference Label',
              ),
              controller: referenceController,
            ),
            SizedBox(height: 15),
            ElevatedButton(
              onPressed: () => setState(() {
                WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
                  Scrollable.ensureVisible(
                    _key.currentContext,
                    duration: Duration(milliseconds: 500),
                  );
                });
              }),
              child: Text('Encode'),
            ),
            QrImage(
              key: _key,
              data: encoded,
              version: QrVersions.auto,
              size: 300.00,
            ),
            ..._divider,
            Text(
              'The encoded QR string:',
              style: TextStyle(color: Colors.deepOrange, fontSize: 16),
            ),
            SelectableText(encoded),
            ..._divider,
            Text(
              'Information of QR code:',
              style: TextStyle(color: Colors.deepOrange, fontSize: 16),
            ),
            Align(
              alignment: Alignment.centerLeft,
              child: SelectableText(qrInfo.toString()),
            ),
            SizedBox(height: 15),
          ]),
        ),
      ),
    );
  }
}
