import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:hkqr_fps/hkqr_fps.dart';

class QrStringDecodePage extends StatefulWidget {
  const QrStringDecodePage({Key key}) : super(key: key);

  @override
  _QrStringDecodePageState createState() => _QrStringDecodePageState();
}

class _QrStringDecodePageState extends State<QrStringDecodePage> {
  static const maxHistoryCount = 10;

  static List<MapEntry<String, DateTime>> history = [];

  void addToHistory(String qrString) {
    if (qrString == null || qrString.isEmpty) return;
    final entry = history.firstWhere((e) => e.key == qrString, orElse: () => null);
    history.remove(entry);
    if (history.length >= maxHistoryCount) history.removeLast();
    history.insert(0, MapEntry(qrString, DateTime.now()));
  }

  TextEditingController qrStringController;

  @override
  void initState() {
    super.initState();
    qrStringController = TextEditingController();
  }

  @override
  void dispose() {
    qrStringController.dispose();
    super.dispose();
  }

  List<Widget> get _divider => [
        SizedBox(height: 15),
        Divider(height: 2, color: Theme.of(context).primaryColor),
        SizedBox(height: 15),
      ];

  String hhMMss(DateTime time) {
    return '${time.hour.toString().padLeft(2, '0')}'
        ':${time.minute.toString().padLeft(2, '0')}'
        ':${time.second.toString().padLeft(2, '0')}';
  }

  Widget _buildHistoryItem(MapEntry<String, DateTime> entry) {
    return GestureDetector(
      child: Row(
        children: [
          Text('${hhMMss(entry.value)}:    '),
          Expanded(child: Text(entry.key, maxLines: 1, overflow: TextOverflow.ellipsis)),
        ],
      ),
      onTap: () => setState(() => qrStringController.text = entry.key),
    );
  }

  @override
  Widget build(BuildContext context) {
    final width = MediaQuery.of(context).size.width;
    return Scaffold(
      appBar: AppBar(title: Text('Decode QR String')),
      body: SingleChildScrollView(
        child: Padding(
          padding: EdgeInsets.symmetric(horizontal: kIsWeb ? width * 0.3 : 36),
          child: Column(children: [
            SizedBox(height: 15),
            Stack(
              children: [
                TextField(
                  maxLines: 6,
                  decoration: InputDecoration(
                    border: OutlineInputBorder(),
                    labelText: 'QR String',
                  ),
                  controller: qrStringController,
                ),
                Positioned(
                  right: 0,
                  bottom: 0,
                  child: IconButton(
                    icon: Icon(Icons.clear, color: Colors.red),
                    onPressed: () => setState(() => qrStringController.text = ''),
                  ),
                ),
              ],
            ),
            ..._divider,
            if (qrStringController.text?.isNotEmpty ?? false) ...[
              Align(
                alignment: Alignment.centerLeft,
                child: Text(HKQrInfo.decode(qrStringController.text).toString()),
              ),
              ..._divider,
            ],
            ElevatedButton(
              onPressed: () {
                FocusScope.of(context).unfocus();
                setState(() => addToHistory(qrStringController.text));
              },
              child: Text('Decode'),
            ),
            if (history.isNotEmpty) ...[
              ..._divider,
              Text(
                'Decode History',
                style: TextStyle(color: Colors.deepOrange, fontSize: 16),
              ),
            ],
            ...history.map((e) => _buildHistoryItem(e)).toList(),
            SizedBox(height: 15),
          ]),
        ),
      ),
    );
  }
}
