import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:hkqr_fps/hkqr_fps.dart';
import 'package:qr_code_scanner/qr_code_scanner.dart';

class ScanAndDecodePage extends StatefulWidget {
  const ScanAndDecodePage({Key key}) : super(key: key);

  @override
  _ScanAndDecodePageState createState() => _ScanAndDecodePageState();
}

class _ScanAndDecodePageState extends State<ScanAndDecodePage> {
  final GlobalKey qrKey = GlobalKey(debugLabel: 'QR');
  Barcode result;
  QRViewController controller;

  // In order to get hot reload to work we need to pause the camera if the platform
  // is android, or resume the camera if the platform is iOS.
  @override
  void reassemble() {
    super.reassemble();
    if (Platform.isAndroid) {
      controller.pauseCamera();
    } else if (Platform.isIOS) {
      controller.resumeCamera();
    }
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    controller?.dispose();
    super.dispose();
  }

  void _onQRViewCreated(QRViewController controller) {
    this.controller = controller;
    controller.scannedDataStream.listen((scanData) async {
      result = scanData;
      await controller.pauseCamera();
      setState(() {});
    });
  }

  List<Widget> get _divider => [
        SizedBox(height: 15),
        Divider(height: 2, color: Theme.of(context).primaryColor),
        SizedBox(height: 15),
      ];

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: Text('Decode QR String')),
      body: Padding(
        padding: EdgeInsets.symmetric(horizontal: 36),
        child: SingleChildScrollView(
          child: Column(children: [
            SizedBox(height: 15),
            SizedBox(
              width: 300,
              height: 300,
              child: QRView(key: qrKey, onQRViewCreated: _onQRViewCreated),
            ),
            ..._divider,
            if (result?.code?.isNotEmpty ?? false) ...[
              Align(
                alignment: Alignment.centerLeft,
                child: SelectableText('Result:\n${result.code}'),
              ),
              ..._divider,
              Align(
                alignment: Alignment.centerLeft,
                child: Text(HKQrInfo.decode(result.code).toString()),
              ),
              ..._divider,
              ElevatedButton(
                onPressed: () => controller?.resumeCamera(),
                child: Text('ReScan'),
              ),
            ],
          ]),
        ),
      ),
    );
  }
}
