enum FpsType {
  phone,
  email,
  fpsId,
}

class FpsProxy {
  final FpsType type;
  final String value;

  bool get isPhone => type == FpsType.phone;

  bool get isEmail => type == FpsType.email;

  bool get isFpsId => type == FpsType.fpsId;

  String get typeDesc => isPhone ? 'Phone' : (isEmail ? 'Email' : 'FpsID');

  const FpsProxy(this.type, this.value);

  @override
  String toString() {
    return 'FpsProxy{$typeDesc/$value}';
  }
}

class DecodeResult {
  static const int errorFormatIncompatible = 10;

  static const int errorUnsupportedValue = 11;

  /// only support fps
  static const int errorUnsupportedPaymentOperator = 12;

  static const int errorCrcCheckFailed = 13;

  static const int errorMandatoryMissed = 14;

  /// -1: unrecognized
  /// 0: success
  /// other positive integer: failed, with a description in [failedReason]
  final int resultCode;

  final String failedReason;

  final List<String> warnings;

  bool get isUnrecognized => resultCode == -1;

  bool get isSuccess => resultCode == 0;

  bool get isFailed => resultCode > 0;

  const DecodeResult(this.resultCode, {this.failedReason = '', this.warnings = const []});

  const DecodeResult.success({List<String> warnings = const []}) : this(0, warnings: warnings);

  factory DecodeResult.failed({int code = errorUnsupportedValue, String reason = ''}) {
    assert(code > 0);
    return DecodeResult(code, failedReason: reason);
  }

  const DecodeResult.unrecognized() : this(-1);

  String get codeDescription {
    switch (resultCode) {
      case errorFormatIncompatible:
        return 'Format Incompatible';
      case errorUnsupportedValue:
        return 'Unsupported Value';
      case errorUnsupportedPaymentOperator:
        return 'Unsupported Payment Operator';
      case errorCrcCheckFailed:
        return 'Crc Check Failed';
      case errorMandatoryMissed:
        return 'Mandatory Missed';
      case 0:
        return 'Success';
      default:
        return '${resultCode > 0 ? 'Unknown Error' : 'Unrecognized'}';
    }
  }
}
